
#ifndef __FILTERS__
#define __FILTERS__

#define F_IBE 1		/* Input buffer empty */
#define F_OBF 2		/* Output buffer full */
#define F_EOF 4
#define F_ERR 8

#define F_INIT 16
#define F_RECONF 32	/* sub_request sending data has changed */

#define F_BUFF 64	/* buffer has changed */
#define F_NINPUT 128	/* cannot be the first (input) filter */
/* Apache specific signals */
#define F_CHARSET 1024	/* charset has changed due to <META> or <mycharset> */

#define F_SIGNAL  (F_EOF|F_ERR|F_RECONF|F_CHARSET)

#define F_INIT_SIGNAL(f)  ( F_INIT | ((f)->flags & F_SIGNAL) )
#define F_IBE_SIGNAL(f) \
	( !((f)->flags & F_SIGNAL) ? F_IBE : \
	      (f)->buf_top != (f)->buf ? 0 : \
		  F_IBE|((f)->flags & F_SIGNAL) )
#define F_OBF_SIGNAL(f) \
	( (f)->buf_bottom == (f)->buf && ((f)->flags & F_SIGNAL) ? \
	      F_IBE|((f)->flags & F_SIGNAL) : F_OBF )
#define F_EOF_SIGNAL(f)   F_EOF
#define F_ERR_SIGNAL(f)   F_ERR
#define F_DONE_SIGNAL(f) \
	( (f)->buf_bottom < (f)->buf_top ? F_OBF : \
	      (f)->buf_top == (f)->buf ? \
		  F_IBE|((f)->flags & F_SIGNAL) : \
		  (f)->attached > 0 ? F_OBF : \
		      ((f)->flags & F_SIGNAL) || !(f)->prev ? \
			  0 : F_IBE )

#define REJECT_SPECIAL_F_SIGNALS(f)  \
	{ if (((f)->flags & F_SIGNAL) && !((f)->flags & (F_EOF|F_ERR))) \
	      return F_INIT|((f)->flags & F_SIGNAL); }
#define CANNOT_BE_OUTPUT_FILTER(f)   \
	{ if (!(f)->next)  return F_ERR_SIGNAL(f); }
#define CANNOT_BE_INPUT_FILTER(f)  \
	{ if (!(f)->prev)  return F_ERR_SIGNAL(f); \
	  else  (f)->flags |= F_NINPUT; }
#define filter_eof(f) \
	( ((f)->flags & (F_EOF|F_ERR)) && (f)->buf_top == (f)->buf )

#ifndef CONV_CHARSET_UTF8
#include <charconv.h>
#endif

typedef struct filter filter;
typedef struct filter_chain filter_chain;

struct filter {
  filter *prev;
  filter *next;
  char *buf, *buf_bottom, *buf_top, *buf_end;
  char *orig_buf, *orig_buf_end;	/* attach_filter_buffer() muze kvuli
		   urychleni filtry ztotoznit (aby se nemusela kopirovat data,
		   ktera se nemeni). Zde je puvodni buf vytvoreny pri
		   add_filter(), aby se nezapomnel. */
  int (*handler)(filter *f, char **src, char *se, char **dst, char *de);
		/* src, se, dst a de  povazuji za pouhe zkratky pro
		   &f->buf_bottom, f->buf_top, &f->next->buf_top a
		   f->next->buf_end, a podle toho s nimi i zachazim!
		        Je to OK? */
  filter_chain *fch;
  int flags;	/* obsahuje signaly pro filtr, aby se pamatovaly, kdyz cely 
		   chain vrati F_OBF; muze mit nastaveno 0,F_EOF,F_RECONF a
		   F_ERR; 0 znamena init */
  int attached;	/* Buffer of this filter is attached to the next buffer. */
  const char *name;
};

struct filter_chain {
  filter *first_filter;
  filter *last_filter;
  int flags;	/* contains F_IBE, F_OBF, F_EOF, F_ERR or F_INIT */

  pool *pool;
  request_rec *r;
};


#define DEFAULT_FILTER_BUFSIZE 4096


extern filter_chain * make_filter_chain (pool *p);
extern void reset_filter_chain (filter_chain *fch);
extern filter *add_filter (filter_chain *fch,
	int (*handler)(filter *f, char **src, char *se, char **dst, char *de),
	long bufsize,
	int f_structsize,
	const char *name);
extern void print_filter_info (filter *f);
extern void print_filter_chain_info (filter_chain *fch);

extern void change_filter_buffer (filter *f, const char *buf, const char *buf_bottom, const char *buf_top, const char *buf_end);
extern void set_filter_buffer (filter *f, const char *buf, int len);
extern void reset_filter_buffer (filter *f);
extern void attach_filter_buffer (filter *f, int minsize);
extern void relocate_filter_buffer (filter *f, const char *buf, const char *buf_bottom, const char *buf_top, const char *buf_end);

extern int advance_filter_chain (filter_chain *fch);
extern int run_filter_chain (filter_chain *fch);
extern int reconf_filter_chain (filter_chain *fch);
extern int flush_filter_chain (filter_chain *fch);

extern int copy_filter_buffer (filter *f, char **src, char *se, char **dst, char *de);

extern int f_dummy_output (filter *f, char **src, char *se, char **dst, char *de);
extern int f_copy_input (filter *f, char **src, char *se, char **dst, char *de);
extern int f_alloc_copy_output (filter *f, char **src, char *se, char **dst, char *de);
extern int f_url_encode (filter *f, char **src, char *se, char **dst, char *de);
extern int f_url_decode (filter *f, char **src, char *se, char **dst, char *de);
extern int f_file_input (filter *f, char **src, char *se, char **dst, char *de);
extern int f_file_output (filter *f, char **src, char *se, char **dst, char *de);
extern int f_stdin_input (filter *f, char **src, char *se, char **dst, char *de);
extern int f_stdout_output (filter *f, char **src, char *se, char **dst, char *de);
extern int f_rwrite_output (filter *f, char **src, char *se, char **dst, char *de);
extern int f_client_output (filter *f, char **src, char *se, char **dst, char *de);
extern int f_client_input (filter *f, char **src, char *se, char **dst, char *de);
extern int f_out_chunk (filter *f, char **src, char *se, char **dst, char *de);
extern int f_charset (filter *f, char **src, char *se, char **dst, char *de);
extern int f_out_charset (filter *f, char **src, char *se, char **dst, char *de);
extern int f_mod_czech (filter *f, char **src, char *se, char **dst, char *de);

typedef struct {
  filter f;
  char *outbuf; 	/* buffer, where the input is copied to */
  int cnt, size;	/* position in outbuf, size of oubuf */
} alloc_copy_filter;

typedef struct {
  filter f;
  unsigned char state[4];	/* used in f_url_decode() */
  int backslash_flag;    	/* used in f_url_encode() */
} url_encoding_filter;

typedef struct {
  filter f;
  char *filename;
  FILE *file;
} file_filter;

typedef struct {
  filter f;
  conn_rec *connection;
} rwrite_filter;

typedef struct {
  filter f;
  int chunked;
} chunk_filter;

typedef struct {
  filter f;
  struct conv_context cc;
  int from, to;
} charset_filter;

#define TAG_BUFF_SIZE 2000

typedef struct {
  charset_filter f;
  char tag_buf[TAG_BUFF_SIZE];	/* Buffer pro tagy */
  char *tag, *c, *top;
  char *tag_bottom;		/* Prvni dosud nezapsany znak v tag_buf */
  char *tag_top;		/* Konec tagu v tag_buf */
  int  state;			/* Stav parseru HTML */
  int  interesting;		/* Prave zpracovavany tag je pro nas zajimavy */
  int  is_html;
} mod_czech_filter;


#endif
