/*
 *	Character Set Recoding and Czech Support Module for Apache Web Server
 *
 *	(c) 1996-1998 Jakub Dadak <smdad@fce.vutbr.cz>
 *	(c) 1997-1998 Martin Mares <mj@atrey.karlin.mff.cuni.cz>
 *	(c) 1997-1998 Martin Plechsmid <plechsmi@karlin.mff.cuni.cz>
 *
 *	Home page: http://www.intersoft.cz/home/cestina.htm
 *
 *	This program can be freely used and distributed according to the terms
 *	of the GNU General Public License version 2 as present in file COPYING
 *	accompanying this package.
 */

#ifndef __MOD_CZECH__
#define __MOD_CZECH__ 

extern module MODULE_VAR_EXPORT czech_module;

int czech_post_read(request_rec *r);
void SetupModCzechEncoding(request_rec *r);
int DecideWhetherEncode (request_rec *r);
int IsContentTypeForEncoding(request_rec *r, const char *content_type);
int find_charset(const char *cp);
char *AddPrefixToURL(request_rec *r, char *url);

typedef struct mcz_config {
  struct mcz_config *master_cf;		/* Konfigurace pro master request */
  int czcode;				/* Zda se koduje nebo ne */
  int czsource;				/* Kodova stranka zdrojove sady */
  int form_cp;				/* Kodova stranka pro formulare */ 
  int form_source;			/* Zdrojova kodova stranka pro formulare */
  int add_prefix;			/* Zda se pridavaji prefixy */
  int mycharset;			/* Kodova stranka nastavena pres <!--mycharset=... */

  /* Prekodovavani vystupu */
  int is_html;				/* Content-type je text/html (tj. upravuji tagy) */

  /* Prekodovavani vstupu */
  int is_form;				/* Jedna se o FORM POST */
  struct filter *first_filter, *last_filter; /* Sekvence vstupnich filtru */
} mcz_config;

typedef struct {			/* Per-server konfigurace */
  table *czprefixes;			/* Prefixy pro explicitni nastaveni cestiny */
  table *charset_aliases;		/* Aliasy jmen charsetu */
  table *postfix_aliases;		/* Postfixy fungujici jako aliasy pro prefixy */
} czech_server_config;

typedef struct {			/* Per-directory konfigurace */
  int encode;				/* Prekodovavame? */
  int form_encode;			/* Prekodovavame FORMy? */
  int url_encode;			/* Prekodovavame parametry v URL? */
  int czsource;				/* Zdrojova CP */ 
  int form_cp;				/* Cilova CP pro z formulare */
  int prepend_prefix;			/* Pridavame prefixy? */
  int do_cache;			        /* Cachovat http/1.0 dokumenty? */
  int set_lm;				/* Nastavujeme Last-modified? */
  int add_cp_to_ct;			/* Pridavame charset k content-type? */
  int allow_dyn_cp_changes;		/* Podporujeme zmenu charsetu pres czsource tag? */
  int guess_charset;			/* Jsme ochotni hadat charset dle jmena browseru */
  int fallback_charset;			/* Co pouzit, kdyz vse selze (-1=poslat redirect na CHCP) */
  int allow_html_parsing;		/* Mame dovoleno hrabat se v HTML? */
  table *src_charsets;                  /* Charsety prislusejici extenzim (pro mime) */ 
  table *content_types;			/* Podporovane content_typy */
  table *czech_servers;			/* Servery s mod_czech */
} czech_dir_config;

/* Lokalni data mod_czech pro kazdy request */
#define CZ_STATE(r) ((mcz_config*)(r)->mcz_cfg)
#define CZ CZ_STATE(r)

/* Per-directory a per-server konfigurace modulu */
#define CZ_DIR_CONFIG(r) ((czech_dir_config *) ap_get_module_config((r)->per_dir_config, &czech_module))
#define CZ_CONFIG(r) ((czech_server_config *) ap_get_module_config((r)->server->module_config, &czech_module))

/* veci specificke pro Ceskou republiku */
#define CZECH_HACKS

#endif
